//**********************************************
//
// FILE : Sample.C
//
// DESCRIPTION : A simple example of reading data from and
// writing data to a MODBUS slave decvice from two threads using WAModbus.DLL
//
//**********************************************


#include <stdio.h>
#include <time.h>
#include <windows.h>
#include <conio.h>
#include <winbase.h>
#include "WAModbus.h"


// Here are some possible internal data addresses you may want to use.
// These correspond to memory locations inside the MODBUS device.
////////////////////////////
// ANAFAZE CLS/MLS parameter addresses
#define CLS_PV		 40364		// CLS Process Variable MODBUS address
#define CLS_SETPOINT 40331		// CLS Set Point MODBUS address
// WATLOW F4 parameter addresses
#define F4_PV		 40101		// F4 Input Reading MODBUS address
#define F4_SETPOINT  40301		// F4 Set Point MODBUS address


////////////////////////////
// Structure to store setting relevant to Reading or Writing daa
typedef struct tagModBusTag
{
	short slaveAddress;		// comm line address, range: 1 to 247
	long startAddress;		// Memory address of parameter in device range:1-499999
	short numPoints;		// # consequetive items to read or write from startAddress
	short data[MAX_MODBUS_DATA_LENGTH-9];	// array containing data read or data to write
	short errorCode;		// MODBUS return error code
	short function;			// For DLL internal use only. Will be set to appropriate MODBUS function code
}ModBusBag;



//////////////////////////////
// Communication port settings
typedef struct tagPortSetup
{
	char portStr[80];		// "COM1", "COM2", or other device name.
	long baudRate;			// 2400, 9600, 19200, etc.
	short parity;			// 0=None, 1=Odd, 2=Even
}PortSetup;


///////////////////////////////////////////////////////////
// The communication port must be referred to with a HANDLE
HANDLE portHandle;

////////////////////////////////////////////////////
// Variables necessary for launching separate thread
HANDLE hPollThread;
DWORD dwThreadID=0;
DWORD passedParm=13;
BOOL pollProcComplete=FALSE;


//////////////////////////////////////////////
// Interpret response from controller for user.
// Error code or valid data may be contained in ModBusBag struc
void printResponse(ModBusBag * mb)
{
	int j;

	if (mb->errorCode!=0)
	{
		printf(" HOST RECEIVED MODBUS ERROR FROM SLAVE : ");
		switch(mb->errorCode)
		{
		case HOST_RECEIVED_IO_ERR_SENDING:
			printf("HOST_RECEIVED_IO_ERR_SENDING\n");
			return;
		case ILLEGAL_FUNCTION:
			printf("ILLEGAL_FUNCTION\n");
			return;;
		case ILLEGAL_DATA_ADDRESS:
			printf("ILLEGAL_DATA_ADDRESS\n");
			return;
		case ILLEGAL_DATA_VALUE:
			printf("ILLEGAL_DATA_VALUE\n");
			return;
		case SLAVE_DEVICE_FAILURE:
			printf("SLAVE_DEVICE_FAILURE\n");
			return;
		case ACKNOWLEDGE:
			printf("ACKNOWLEDGE\n");
			return;
		case SLAVE_DEVICE_BUSY:
			printf("SLAVE_DEVICE_BUSY\n");
			return;
		case NEGATIVE_ACKNOWLEDGE:
			printf("NEGATIVE_ACKNOWLEDGE\n");
			return;
		case MEMORY_PARITY_ERROR:
			printf("MEMORY_PARITY_ERROR\n");
			return;
		case HOST_RECEIVED_TIMEOUT_ERROR:
			printf("HOST RECEIVED TIMEOUT ERROR\n");
			return;
		case HOST_RECEIVED_CHECKSUM_ERROR:
			printf("HOST RECEIVED CHECKSUM ERROR\n");
			return;
		case HOST_RECEIVED_MODBUS_ERROR:
			switch(mb->data[0])
			{
			case ILLEGAL_FUNCTION : printf("ILLEGAL_FUNCTION\n");
				break;
			case ILLEGAL_DATA_ADDRESS : printf("ILLEGAL_DATA_ADDRESS\n");
				break;
			case ILLEGAL_DATA_VALUE : printf("ILLEGAL_DATA_VALUE\n");
				break;
			}
			return;
		}
	}
	if (mb->numPoints > 0)
	{
		for (j=0; j<mb->numPoints; j++)
			printf(" %d", mb->data[j]);
		putch('\n');
	}
}




////////////////////////////////////////////
// This thread reads an F4s Process Variable 
// and writes that value to the F4s Set Point register
BOOL pollProc(DWORD *parmPtr)
{
     ModBusBag mbRead;			// Read/Write data structures

	 pollProcComplete=FALSE;	// Keep polling until main() loop says stop

	 mbRead.slaveAddress = 1;	// Be sure to set the controller's communication address
	 mbRead.numPoints = 1;		// For multiloop controllers this may be > 1
     do
     {
		mbRead.startAddress = F4_PV;
		mbRead.errorCode = WA_readModbusDevice(portHandle, mbRead.slaveAddress, mbRead.startAddress, mbRead.data, mbRead.numPoints);      // Read from device
		printf("Thread #2: Reading Process Variable:");
		printResponse(&mbRead);

		mbRead.startAddress = F4_SETPOINT;
		mbRead.errorCode = WA_readModbusDevice(portHandle, mbRead.slaveAddress, mbRead.startAddress, mbRead.data, mbRead.numPoints);      // Read from device
		printf("Thread #2: Writing Set Point:");
		printResponse(&mbRead);
     }
     while(!kbhit());


	hPollThread=0;
	dwThreadID=0;
	return TRUE;
}

BOOL startThread(void)
{
     if (NULL == (hPollThread = CreateThread( (LPSECURITY_ATTRIBUTES) NULL, 0, (LPTHREAD_START_ROUTINE) pollProc,
                                    (LPVOID) &passedParm, 0, &dwThreadID )))
	 {
		 return FALSE;
	 }

	 return TRUE;
}


///////////////////////////////////////////////////////////////////
// The program reads a Process Variable and writes it to the Set Point
void main(int argc, char* argv[])
{
     PortSetup ps;                // Comm port setup data struct
     ModBusBag mbRead;			  // Read/Write data structs


	// Set Modbus communicaitons Baudrate
     ps.baudRate = 19200;
	 // Set the communication port to use.
     strcpy(ps.portStr, "COM2");
     ps.parity = 0;

	 // Open COM port here
	 if (WA_openModbusConnection(&portHandle, ps.portStr, ps.baudRate, ps.parity) != GOOD)
     {
          printf("\nCan not open port %s!\n", ps.portStr);
          return;
     }

	 startThread();		// illustrate DLL multi-threading capability by starting thread procedure

	
	 ///////////////////////////////////////
	// Set the slave's communication address
	mbRead.slaveAddress = 1;
	/////////////////////////////////////////////////////////////////////////////////////
	// Set the number of points to up/download, for multiloop controllers this may be > 1
	mbRead.numPoints = 1;		

	 // The following loop continually reads the Process Variable value from a device and writes that same value to the Set Point parameter.
     do
     {
			mbRead.startAddress = F4_PV;
			mbRead.errorCode = WA_readModbusDevice(portHandle, mbRead.slaveAddress, mbRead.startAddress, mbRead.data, mbRead.numPoints);      // Read from device
			printf("Thread #1: Reading Process Variable:");
			printResponse(&mbRead);

			mbRead.startAddress = F4_SETPOINT;
			mbRead.errorCode = WA_readModbusDevice(portHandle, mbRead.slaveAddress, mbRead.startAddress, mbRead.data, mbRead.numPoints);      // Read from device
			printf("Thread #1: Writing Set Point:");
			printResponse(&mbRead);
     }
     while(!kbhit());

	 pollProcComplete=TRUE;

	 while(dwThreadID);

	 // Close COM port here
     WA_closeModbusConnection(portHandle);
}

